@@ -5,7 +5,7 @@ use 5.008_001;
 use strict;
 use warnings FATAL => 'recursion';
 
-our $VERSION = '0.1003';
+our $VERSION = '0.1004';
 
 use Any::Moose;
 
@@ -148,6 +148,11 @@ our %loop_context_vars = (
     __last__    => \&iterator_last,
 );
 
+has input_filter => (
+    is => 'rw',
+    isa => 'CodeRef',
+);
+
 has parser => (
     is       => 'rw',
     required => 1,
@@ -238,6 +243,8 @@ sub _build_op_to_type_table {
 sub parse {
     my($self, $input, %no_use) = @_;
 
+    $self->input_filter->(\$input) if($self->input_filter);
+
     my $tree = $self->parser->parse($input);
     my @ast = $self->tree_to_ast($tree);
     \@ast;
@@ -370,7 +377,13 @@ sub convert_include {
     my $include = $self->generate_include($self->convert_name_or_expr($node->name_or_expr));
 
     if($self->loop_depth){
-        $include->second([ $self->generate_variable('$'.$self->dummy_loop_item_name.$self->loop_depth) ]);
+        $include->second([
+            $self->generate_methodcall(
+                '.',
+                $self->generate_vars('__ROOT__'),
+                $self->generate_literal(undef, 'merge'),
+                $self->generate_variable('$'.$self->dummy_loop_item_name.$self->loop_depth),
+            )]);
     }
     $include;
 }
@@ -563,7 +576,15 @@ sub generate_literal {
     my($self, $value, $id) = @_;
 
     $id = $value if(not defined $id);
-    Text::Xslate::Symbol->new(arity => 'literal', id => $id, value => $value);
+    my $literal = Text::Xslate::Symbol->new(arity => 'literal', id => $id);
+    $literal->value($value) if defined $value;
+    $literal;
+}
+
+sub generate_vars {
+    my($self, $name) = @_;
+
+    Text::Xslate::Symbol->new(arity => 'vars', id => $name);
 }
 
 sub generate_variable {
@@ -596,6 +617,14 @@ sub generate_call {
     Text::Xslate::Symbol->new(arity => 'call', id => '(', first => $self->generate_name($name), second => $args);
 }
 
+sub generate_methodcall {
+    my($self, $name, $first, $second, @other_args) = @_;
+
+    Text::Xslate::Symbol->new(arity => 'methodcall', id => $name, first => $first, second => $second, third => \@other_args);
+}
+
+
+
 sub generate_binary {
     my($self, $id, $first, $second) = @_;
     Text::Xslate::Symbol->new(
